/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.client;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import javax.xml.namespace.QName;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import uk.org.ogsadai.common.xml.XMLUtilities;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceRequest;
import uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceResponse;
import uk.org.ogsadai.dais.xml.service.core.GetDataResourcePropertyDocumentRequest;
import uk.org.ogsadai.dais.xml.service.core.GetResourceListRequest;
import uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.service.core.ResolveRequest;
import uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentRequestWrapper;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.AddSchemaRequest;
import uk.org.ogsadai.dais.xml.services.daix.AddSchemaResponse;
import uk.org.ogsadai.dais.xml.services.daix.CollectionAlreadyExistsFaultType;
import uk.org.ogsadai.dais.xml.services.daix.CollectionSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.DocumentDoesNotExistFaultType;
import uk.org.ogsadai.dais.xml.services.daix.DocumentSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentRequestWrapper;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.GetItemsRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetItemsResponse;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCountFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidStartPositionFaultType;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentRequestWrapper;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.SchemaAdditionMakesDocumentsInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaDoesNotExistFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaRemovalMakesDocumentsInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XMLSerializationParameters;
import uk.org.ogsadai.dais.xml.services.daix.XMLWrapperType;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XPathExpressionType;
import uk.org.ogsadai.dais.xml.services.daix.XPathFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExpressionType;
import uk.org.ogsadai.dais.xml.services.daix.XQueryFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExpressionType;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateFaultType;
import uk.org.ogsadai.dais.xml.services.daix.bindings.CoreDataAccessSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.CoreResourceListSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XMLCollectionAccessSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XMLCollectionFactorySOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XMLSequenceAccessSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XPathAccessSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XPathFactorySOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XQueryAccessSOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XQueryFactorySOAPStub;
import uk.org.ogsadai.dais.xml.services.daix.bindings.XUpdateAccessSOAPStub;
import uk.org.ogsadai.service.addressing.EndpointReferenceType;
import uk.org.ogsadai.wsdaix.common.XQuerySerializationParametersUtils;
import uk.org.ogsadai.wsdaix.test.XMLCollectionAccessTest;
import uk.org.ogsadai.wsdaix.test.properties.TestDocuments;

/**
 * The Client Toolkit contains a set of static methods for invoking
 * operations on WS-DAIX endpoints. 
 * 
 * @author Steven Lynden
 * 
 */
public class ClientToolkit {

    /**
     * Execute the GetDataResourcePropertyDocument operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resourcee
     * @return The property document
     */
    public static PropertyDocumentType getDataResourcePropertyDocument(String serviceURL, URI resourceName) throws DataResourceUnavailableFaultType, MalformedURLException, InvalidResourceNameFaultType, NotAuthorizedFaultType, RemoteException {
        GetDataResourcePropertyDocumentRequest request = new GetDataResourcePropertyDocumentRequest(); 
        request.setDataResourceAbstractName(resourceName);
        CoreDataAccessSOAPStub stub = new CoreDataAccessSOAPStub(new URL(serviceURL),null);
        return stub.getDataResourcePropertyDocument(request);
    }
    
    /**
     * Execute the GetCollectionPropertyDocument operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @return The XML Collection property document
     */
    public static PropertyDocumentType getCollectionPropertyDocument(String serviceURL, URI resourceName) throws DataResourceUnavailableFaultType, MalformedURLException, ServiceBusyFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType, RemoteException {
        GetDataResourcePropertyDocumentRequest request = new GetDataResourcePropertyDocumentRequest(); 
        request.setDataResourceAbstractName(resourceName);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.getCollectionPropertyDocument(request);
    }
    
    /**
     * Execute the GetXMLSequencePropertyDocument operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @return The XML Sequence property document
     */
    public static PropertyDocumentType getXMLSequencePropertyDocument(String serviceURL, URI resourceName) throws DataResourceUnavailableFaultType, MalformedURLException, InvalidResourceNameFaultType, RemoteException {
        GetDataResourcePropertyDocumentRequest request = new GetDataResourcePropertyDocumentRequest(); 
        request.setDataResourceAbstractName(resourceName);
        XMLSequenceAccessSOAPStub stub = new XMLSequenceAccessSOAPStub(new URL(serviceURL),null);
        return stub.getXMLSequencePropertyDocument(request);
    }
    
    /**
     * Execute the Resolve operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @return An array of data resource addresses
     */
    public static DataResourceAddressType[] resolve(String serviceURL, URI resourceName) throws MalformedURLException, InvalidResourceNameFaultType, NotAuthorizedFaultType, RemoteException {
        ResolveRequest request = new ResolveRequest();
        request.setDataResourceAbstractName(resourceName);
        CoreResourceListSOAPStub stub = new CoreResourceListSOAPStub(new URL(serviceURL),null);
        return stub.resolve(request).getDataResourceAddress();
    }
    
    /**
     * Execute the DestoryDataResource operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @return A response indicates success in destroying the resource
     */
    public static DestroyDataResourceResponse destroyDataResource(String serviceURL, URI resourceName) throws DataResourceUnavailableFaultType, MalformedURLException, InvalidResourceNameFaultType, NotAuthorizedFaultType, RemoteException {
        DestroyDataResourceRequest request = new DestroyDataResourceRequest();
        request.setDataResourceAbstractName(resourceName);
        CoreDataAccessSOAPStub stub = new CoreDataAccessSOAPStub(new URL(serviceURL),null);
        return stub.destroyDataResource(request);
    }
    
    /**
     * Execute the GetResourceList operation.
     * @param serviceURL The URL of the data service
     * @return An array of data resource addresses
     */
    public static DataResourceAddressType[] getResourceList(String serviceURL) throws MalformedURLException, NotAuthorizedFaultType, RemoteException {
        GetResourceListRequest request = new GetResourceListRequest();
        CoreResourceListSOAPStub stub = new CoreResourceListSOAPStub(new URL(serviceURL),null);
        return stub.getResourceList(request).getDataResourceAddress();
    }
    
    /**
     * Execute the DocumentSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param document The name of the document to be selected
     * @return Address of the created data resource
     */
    public static DataResourceAddressType documentSelectionFactory(String serviceURL, URI resourceName, String document) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, DocumentDoesNotExistFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return documentSelectionFactory(serviceURL,resourceName,null,document,null,null);   
    }
    
    /**
     * Execute the DocumentSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param document The name of the document to be selected
     * @param collectionName The URI of a collection in which the document resides 
     * @return Address of the created data resource
     */
    public static DataResourceAddressType documentSelectionFactory(String serviceURL, URI resourceName, URI collectionName, String document) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, DocumentDoesNotExistFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return documentSelectionFactory(serviceURL,resourceName,collectionName,document,null,null);   
    }
    
    /**
     * Execute the DocumentSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param document The name of the document to be selected
     * @param collectionName The URI of a collection in which the document resides
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible 
     * @param config A configuration document used to configure the created data resource
     * @return Address of the created data resource
     */
    public static DataResourceAddressType documentSelectionFactory(String serviceURL, URI resourceName, URI collectionURI, String document, QName portTypeQName, ConfigurationDocumentType config) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, DocumentDoesNotExistFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        DocumentSelectionFactoryRequest request = new DocumentSelectionFactoryRequest();
        request.setConfigurationDocument(config);
        request.setDataResourceAbstractName(resourceName);
        request.setPortTypeQName(portTypeQName);
        request.setDocumentName(document);
        if ( collectionURI != null ) 
        	request.setCollectionName(collectionURI);
        XMLCollectionFactorySOAPStub stub = new XMLCollectionFactorySOAPStub(new URL(serviceURL),null);
        return stub.documentSelectionFactory(request);
    }

    /**
     * Execute the CollectionSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @return Address of the created data resource
     */
    public static DataResourceAddressType collectionSelectionFactory(String serviceURL, URI resourceName) throws DataResourceUnavailableFaultType, MalformedURLException, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return collectionSelectionFactory(serviceURL,resourceName,null,null,null);            
    }
    
    /**
     * Execute the CollectionSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The URI of the collection to be selected
     * @return Address of the created data resource
     */
    public static DataResourceAddressType collectionSelectionFactory(String serviceURL, URI resourceName, URI collectionURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return collectionSelectionFactory(serviceURL,resourceName,collectionURI,null,null);            
    }
    
    /**
     * Execute the CollectionSelectionFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The URI of the collection to be selected
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible 
     * @param config A configuration document used to configure the created data resource
     * @return Address of the created data resource
     */
    public static DataResourceAddressType collectionSelectionFactory(String serviceURL, URI resourceName, URI collectionURI, QName portTypeQName, XMLCollectionConfigurationDocumentType config) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        CollectionSelectionFactoryRequest request = new CollectionSelectionFactoryRequest();
        request.setPortTypeQName(portTypeQName);
        request.setConfigurationDocument(config);
        request.setDataResourceAbstractName(resourceName);
        if ( collectionURI != null )
        	request.setCollectionName(collectionURI);
        XMLCollectionFactorySOAPStub stub = new XMLCollectionFactorySOAPStub(new URL(serviceURL),null);
        return stub.collectionSelectionFactory(request);
    }
    
    /**
     * Execute the RemoveDocuments operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param documentNames An array of document names that should be removed
     */
    public static RemoveDocumentsResponse removeDocuments(String serviceURL, URI resourceName, String[] documentNames) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
       return removeDocuments(serviceURL,resourceName,null,documentNames);
    }
    
    /**
     * Execute the RemoveDocuments operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The URI of the collection from which documents should be removed
     * @param documentNames An array of document names that should be removed
     */
    public static RemoveDocumentsResponse removeDocuments(String serviceURL, URI resourceName, URI collectionURI, String[] documentNames) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        RemoveDocumentsRequest request = new RemoveDocumentsRequest();
        request.setDataResourceAbstractName(resourceName);
        RemoveDocumentRequestWrapper[] wrappers = new RemoveDocumentRequestWrapper[documentNames.length];
        for(int i=0;i<wrappers.length;i++) {
            wrappers[i] = new RemoveDocumentRequestWrapper();
            wrappers[i].setDocumentName(documentNames[i]);
        }
        request.setRemoveDocumentRequestWrapper(wrappers);
        request.setCollectionName(collectionURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.removeDocuments(request);
    }
    
    /**
     * Executes the GetDocuments operation.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param documentNames An array of document names to be retrieved
     */
    public static GetDocumentsResponse getDocuments(String serviceURL, URI resourceName, String[] documentNames) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return getDocuments(serviceURL,resourceName,null,documentNames);
    }
    
    /**
     * Executes the GetDocuments operation.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param documentNames An array of document names to be retrieved
     * @param collectionURI The URI of the collection from which documents should be retrieved
     */
    public static GetDocumentsResponse getDocuments(String serviceURL, URI resourceName, URI collectionURI, String[] documentNames) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        GetDocumentsRequest request = new GetDocumentsRequest();
        request.setDataResourceAbstractName(resourceName);
        GetDocumentRequestWrapper[] wrappers = new GetDocumentRequestWrapper[documentNames.length];
        for(int i=0;i<wrappers.length;i++) {
            wrappers[i] = new GetDocumentRequestWrapper();
            wrappers[i].setDocumentName(documentNames[i]);
        }
        request.setGetDocumentRequestWrapper(wrappers);
        request.setCollectionName(collectionURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.getDocuments(request);
    }
    
    /**
     * Execute the AddDocuments operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param documentNames An array of document names to be added
     * @param documents The corresponding content of each document
     */
    public static AddDocumentsResponse addDocuments(String serviceURL, URI resourceName, String[] documentNames, Document[] documents) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
    	return addDocuments(serviceURL,resourceName,null,documentNames,documents);
    }
    
    /**
     * Execute the AddDocuments operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The URI of the collection to which the documents should be added
     * @param documentNames An array of document names to be added
     * @param documents The corresponding content of each document
     */
    public static AddDocumentsResponse addDocuments(String serviceURL, URI resourceName, URI collectionURI, String[] documentNames, Document[] documents) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        AddDocumentsRequest request = new AddDocumentsRequest();
        request.setDataResourceAbstractName(resourceName);
        AddDocumentRequestWrapper[] wrappers = new AddDocumentRequestWrapper[documents.length];
        for(int i=0;i<wrappers.length;i++) {
            wrappers[i] = new AddDocumentRequestWrapper();
            wrappers[i].setDocumentName(documentNames[i]);
            XMLWrapperType wrapper = new XMLWrapperType();
            MessageElement msgElement = new MessageElement(documents[i].getDocumentElement());
            wrapper.set_any(new MessageElement[] {msgElement} );
            wrappers[i].setData(wrapper);
        }
        request.setAddDocumentRequestWrapper(wrappers);
        request.setCollectionName(collectionURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.addDocuments(request);
    }
    
    /**
     * Execute the RemoveSubcollection operation.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param subcollectionURI URI of the collection to be removed
     * @return A response indicates success
     */
    public static RemoveSubcollectionResponse removeCollection(String serviceURL, URI resourceName, URI subcollectionURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return removeCollection(serviceURL,resourceName,null,subcollectionURI);
    }
    
    /**
     * Execute the RemoveSubcollection operation.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The collection containing the subcollection
     * @param subcollectionURI URI of the collection to be removed
     * @return A response indicates success
     */
    public static RemoveSubcollectionResponse removeCollection(String serviceURL, URI resourceName, URI collectionURI, URI subcollectionURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        RemoveSubcollectionRequest request = new RemoveSubcollectionRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setCollectionName(collectionURI);
        request.setSubcollectionName(subcollectionURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.removeSubcollection(request);
    }
    
    /**
     * Execute the RemoveSchema operation.
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The collection from which the schema is to be removed
     * @param namespaceURI The namespace URI of the schema to be removed
     * @return A response indicates success
     */
    public static RemoveSchemaResponse removeSchema(String serviceURL, URI resourceName, URI collectionURI, URI namespaceURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, SchemaRemovalMakesDocumentsInvalidFaultType, ServiceBusyFaultType, SchemaDoesNotExistFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        RemoveSchemaRequest request = new RemoveSchemaRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setCollectionName(collectionURI);
        request.setSchemaNamespace(namespaceURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.removeSchema(request);
    }
    
    /**
     * Execute the AddSchema operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI The collection to which the schema is added
     * @param schema The schema to be added
     */
    public static AddSchemaResponse addSchema(String serviceURL, URI resourceName, URI collectionURI, Element schema) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, SchemaInvalidFaultType, ServiceBusyFaultType, SchemaAdditionMakesDocumentsInvalidFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        AddSchemaRequest request = new AddSchemaRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setCollectionName(collectionURI);
        request.set_any(new MessageElement[] {new MessageElement(schema)});
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.addSchema(request);
    }
    
    /**
     * Execute the CreateSubcollection operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param subcollectionURI The URI of the collection to be created
     * @return A response indicated success
     */
    public static CreateSubcollectionResponse createCollection(String serviceURL, URI resourceName, URI subcollectionURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, CollectionAlreadyExistsFaultType, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        return createCollection(serviceURL,resourceName,null,subcollectionURI);
    }
    
    /**
     * Execute the CreateSubcollection operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param collectionURI URI of the collection to which the subcollection should be added
     * @param subcollectionURI The URI of the collection to be created
     * @return A response indicated success
     */
    public static CreateSubcollectionResponse createCollection(String serviceURL, URI resourceName, URI collectionURI, URI subcollectionURI) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, CollectionAlreadyExistsFaultType, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType {
        CreateSubcollectionRequest request = new CreateSubcollectionRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setCollectionName(collectionURI);
        request.setSubcollectionName(subcollectionURI);
        XMLCollectionAccessSOAPStub stub = new XMLCollectionAccessSOAPStub(new URL(serviceURL),null);
        return stub.createSubcollection(request);
    }
    
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XQueryExecuteFactory(serviceURL,resourceName,expression,null,null,null,null);   
    }
    
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XQueryExecuteFactory(serviceURL,resourceName,expression,language,null,null,null);   
    }
    
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     * @param configDocument A configuration document used to configure the created data resource
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression, QName portTypeQName, ConfigurationDocumentType configDocument) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XQueryExecuteFactory(serviceURL,resourceName,expression,null,portTypeQName,configDocument,null);
    }
 
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression, QName portTypeQName) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XQueryExecuteFactory(serviceURL,resourceName,expression,null,portTypeQName,null,null);
    }
    
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language, QName portTypeQName) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XQueryExecuteFactory(serviceURL,resourceName,expression,language,portTypeQName,null,null);
    }
    
    /**
     * Execute the XQueryExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     * @param configDocument A configuration document used to configure the created data resource
     * @param preferredTargetService The preferred service through which the resource should be accessible
     */
    public static DataResourceAddressType XQueryExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language, QName portTypeQName, ConfigurationDocumentType configDocument, EndpointReferenceType preferredTargetService ) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        XQueryExecuteFactoryRequest request = new XQueryExecuteFactoryRequest();
        request.setDataResourceAbstractName(resourceName);
        XQueryExpressionType exp = new XQueryExpressionType();
        MessageElement expressionElem = new MessageElement(
                expression);
        exp.set_any(new MessageElement[] { expressionElem } );
        request.setXQueryExpression(exp);
        exp.setLanguage(language);
        XQueryFactorySOAPStub stub = new XQueryFactorySOAPStub(new URL(serviceURL),null);
        if ( portTypeQName != null )
            request.setPortTypeQName(portTypeQName);
        if ( configDocument != null )
            request.setConfigurationDocument(configDocument);
        if ( preferredTargetService != null )
            request.setPreferredTargetService(preferredTargetService);
        return stub.XQueryExecuteFactory(request);
    }
    
    /**
     * Execute the GetItems operation 
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param startPosition An integer specifying the first element of the sequence that should be returned
     * @param count The number of elements to be returned
     */
    public static GetItemsResponse getItems(String serviceURL, URI resourceName, long startPosition, long count) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidStartPositionFaultType, ServiceBusyFaultType, InvalidCountFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        return getItems(serviceURL,resourceName,null,startPosition,count);  
    }
    
    /**
     * Execute the GetItems operation 
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicating the dataset format that should be used to represent results
     * @param startPosition An integer specifying the first element of the sequence that should be returned
     * @param count The number of elements to be returned
     */
    public static GetItemsResponse getItems(String serviceURL, URI resourceName, URI datasetFormatURI, long startPosition, long count) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidStartPositionFaultType, ServiceBusyFaultType, InvalidCountFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XMLSerializationParameters params = XQuerySerializationParametersUtils.constructDefaultSerializationParameters();
        return getItems(serviceURL,resourceName,datasetFormatURI,startPosition,count,params);
    }
    
    /**
     * Execute the GetItems operation 
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicating the dataset format that should be used to represent results
     * @param startPosition An integer specifying the first element of the sequence that should be returned
     * @param count The number of elements to be returned
     * @param xquerySerializationParameters Parameters used to serialize the resulting XML
     */
    public static GetItemsResponse getItems(String serviceURL, URI resourceName, URI datasetFormatURI, long startPosition, long count, XMLSerializationParameters xquerySerializationParameters) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, InvalidStartPositionFaultType, ServiceBusyFaultType, InvalidCountFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        GetItemsRequest request = new GetItemsRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setDatasetFormatURI(datasetFormatURI);
        request.setCount(java.math.BigInteger.valueOf(count));
        request.setStartPosition(java.math.BigInteger.valueOf(startPosition));
        XMLSequenceAccessSOAPStub stub = new XMLSequenceAccessSOAPStub(new URL(serviceURL),null);
        request.setXMLSerializationParameters(xquerySerializationParameters);
        return stub.getItems(request);
    }
    
    /**
     * Execute the XQueryExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted expression
     */
    public static XQueryExecuteResponse XQueryExecute(String serviceURL, URI resourceName, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, XQueryFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        return XQueryExecute(serviceURL,resourceName,null,expression);   
    }
    
    /**
     * Execute the XQueryExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicated how results should be formatted
     * @param expression An XQueryX formatted expression
     */
    public static XQueryExecuteResponse XQueryExecute(String serviceURL, URI resourceName, URI datasetFormatURI, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, XQueryFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XMLSerializationParameters params = XQuerySerializationParametersUtils.constructDefaultSerializationParameters();
        return XQueryExecute(serviceURL,resourceName,datasetFormatURI,expression,params);
    }
    
    /**
     * Execute the XQueryExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicated how results should be formatted
     * @param expression An XQueryX formatted expression
     * @param xquerySerializationParameters Parameters used to serialize the resulting XML
     */
    public static XQueryExecuteResponse XQueryExecute(String serviceURL, URI resourceName, URI datasetFormatURI, Element expression, XMLSerializationParameters xquerySerializationParameters) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, XQueryFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XQueryExecuteRequest request = new XQueryExecuteRequest();
        request.setDataResourceAbstractName(resourceName);
        XQueryExpressionType exp = new XQueryExpressionType();
        MessageElement expressionElem = new MessageElement(expression);
        exp.set_any(new MessageElement[] { expressionElem } );
        request.setXQueryExpression(exp);
        request.setDatasetFormatURI(datasetFormatURI);
        XQueryAccessSOAPStub stub = new XQueryAccessSOAPStub(new URL(serviceURL),null);
        request.setXMLSerializationParameters(xquerySerializationParameters);
        return stub.XQueryExecute(request);
    }
    
    /**
     * Execute the XUpdateExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XUpdate expression
     */
    public static XUpdateExecuteResponse XUpdateExecute(String serviceURL, URI resourceName, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, XUpdateFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
       return XUpdateExecute(serviceURL,resourceName,expression,null);
    }
    
    /**
     * Execute the XUpdateExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XUpdate expression
     * @param language A URI indicating the language used
     */
    public static XUpdateExecuteResponse XUpdateExecute(String serviceURL, URI resourceName, Element expression, URI language) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, XUpdateFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XUpdateExecuteRequest request = new XUpdateExecuteRequest();
        request.setDataResourceAbstractName(resourceName);
        XUpdateExpressionType exprType = new XUpdateExpressionType();
        exprType.set_any( new MessageElement[] {new MessageElement(expression)} );
        exprType.setLanguage(language);
        request.setXUpdateExpression(exprType);
        XUpdateAccessSOAPStub stub = new XUpdateAccessSOAPStub(new URL(serviceURL),null);
        return stub.XUpdateExecute(request);
    }
    
    /**
     * Execute the XPathExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted expression
     */
    public static XPathExecuteResponse XPathExecute(String serviceURL, URI resourceName, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        return XPathExecute(serviceURL,resourceName,null,expression);   
    }
    
    /**
     * Execute the XPathExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicated how results should be formatted
     * @param expression An XQueryX formatted expression
     */
    public static XPathExecuteResponse XPathExecute(String serviceURL, URI resourceName, URI datasetFormatURI, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XMLSerializationParameters params = XQuerySerializationParametersUtils.constructDefaultSerializationParameters();
        return XPathExecute(serviceURL,resourceName,datasetFormatURI,expression,params);
    }
    
    /**
     * Execute the XPathExecute operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param datasetFormatURI A URI indicated how results should be formatted
     * @param expression An XQueryX formatted expression
     * @param xquerySerializationParameters Parameters used to serialize the resulting XML
     */
    public static XPathExecuteResponse XPathExecute(String serviceURL, URI resourceName, URI datasetFormatURI, Element expression, XMLSerializationParameters xquerySerializationParameters) throws DataResourceUnavailableFaultType, MalformedURLException, RemoteException, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        XPathExecuteRequest request = new XPathExecuteRequest();
        request.setDataResourceAbstractName(resourceName);
        request.setDatasetFormatURI(datasetFormatURI);
        XPathExpressionType exp = new XPathExpressionType();
        MessageElement expressionElem = new MessageElement(
                expression);
        exp.set_any(new MessageElement[] { expressionElem } );
        request.setXPathExpression(exp);
        request.setXMLSerializationParameters(xquerySerializationParameters);
        XPathAccessSOAPStub stub = new XPathAccessSOAPStub(new URL(serviceURL),null);
        return stub.XPathExecute(request);
    }
    
    /**
     * Execute the XPathExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     */
    public static DataResourceAddressType XPathExecuteFactory(String serviceURL, URI resourceName, Element expression) throws DataResourceUnavailableFaultType, MalformedURLException,  RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XPathExecuteFactory(serviceURL,resourceName,expression,null,null,null,null);   
    }
    
    /**
     * Execute the XPathExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     */
    public static DataResourceAddressType XPathExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language) throws DataResourceUnavailableFaultType, MalformedURLException,  RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XPathExecuteFactory(serviceURL,resourceName,expression,language,null,null,null);   
    }
    
    /**
     * Execute the XPathExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     */
    public static DataResourceAddressType XPathExecuteFactory(String serviceURL, URI resourceName, Element expression, QName portTypeQName) throws DataResourceUnavailableFaultType, MalformedURLException,  RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XPathExecuteFactory(serviceURL,resourceName,expression,null,portTypeQName,null,null);   
    }
    
    /**
     * Execute the XPathExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     */
    public static DataResourceAddressType XPathExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language, QName portTypeQName) throws DataResourceUnavailableFaultType, MalformedURLException,  RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        return XPathExecuteFactory(serviceURL,resourceName,expression,language,portTypeQName,null,null);   
    }
    
    /**
     * Execute the XPathExecuteFactory operation
     * @param serviceURL The URL of the data service
     * @param resourceName The abstract name of the data resource
     * @param expression An XQueryX formatted query expression
     * @param language A URI specifying the language URI used
     * @param portTypeQName The QName of the port type through which the created data resource should be accessible
     * @param configDocument A configuration document used to configure the created data resource
     * @param preferredTargetService The endpoint reference of the preferred service through which the data resource should be exposed
     */
    public static DataResourceAddressType XPathExecuteFactory(String serviceURL, URI resourceName, Element expression, URI language, QName portTypeQName, ConfigurationDocumentType configDocument, EndpointReferenceType preferredTargetService) throws DataResourceUnavailableFaultType, MalformedURLException,  RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        XPathExecuteFactoryRequest request = new XPathExecuteFactoryRequest();
        request.setDataResourceAbstractName(resourceName);
        XPathExpressionType exp = new XPathExpressionType();
        MessageElement expressionElem = new MessageElement(
                expression);
        exp.set_any(new MessageElement[] { expressionElem } );
        request.setXPathExpression(exp);
        exp.setLanguage(language);
        if ( portTypeQName != null )
            request.setPortTypeQName(portTypeQName);
        if ( configDocument != null )
            request.setConfigurationDocument(configDocument);
        if ( preferredTargetService != null )
            request.setPreferredTargetService(preferredTargetService);
        XPathFactorySOAPStub stub = new XPathFactorySOAPStub(new URL(serviceURL),null);
        return stub.XPathExecuteFactory(request);
    }
    
    /*
     * TODO remove - just for testing changes to the XML collection access fns 
     */
    public static void main(String[] args) {
        try {
            
            XMLCollectionAccessTest t = new XMLCollectionAccessTest();
            t.setUp();
            t.testCreateSubcollection_InvalidCollectionNameFault();
            
            /*
            String collAccURL = "http://rpc122.cs.man.ac.uk:8080/axis/services/CollectionServiceCollectionAccessPT";
            String collFacURL = "http://rpc122.cs.man.ac.uk:8080/axis/services/CollectionServiceCollectionFactoryPT";
            URI resourceName = new URI("daix:testresource");
            URI collection = new URI("daix:/db/contacts/epcc");
            URI newCollection = new URI("jimmy:marshal/bonkers");
            Client c = new Client(collAccURL,resourceName);
            Element e = c.getCollectionPropertyDocument();
            System.out.println( XMLUtilities.xmlDOMToString(e) );
            */
            
            
            
            
            
        } catch ( Exception e ) { System.out.println(e.getClass().getName()); System.out.println(e); }
        
    }
        
}
