/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import java.io.StringReader;

import javax.xml.namespace.QName;

import org.apache.axis.MessageContext;
import org.apache.axis.client.AxisClient;
import org.apache.axis.encoding.DeserializationContext;
import org.apache.axis.encoding.Deserializer;
import org.apache.axis.message.EnvelopeHandler;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.SOAPHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import uk.org.ogsadai.common.xml.XMLUtilities;

/**
 * Utility methods for converting Java <-> XML
 */
public class Serializer {

    /**
     * Convert an XML structure into its equivalent object representation
     */
    public static Object xmlToObject(Document doc, Class javaClass) throws SAXException {
        String is = XMLUtilities.xmlDOMToString(doc);
        InputSource inputSource = new InputSource(new StringReader(is));
        MessageContext ctx = new MessageContext(new AxisClient());
        ctx.setProperty(AxisClient.PROP_DOMULTIREFS,Boolean.TRUE);
        ObjectDeserializationContext deserializer = new ObjectDeserializationContext(inputSource, javaClass, ctx);
        deserializer.parse();
        return deserializer.getValue();
    }
    
    /**
     * Convert an object into its equivalent XML representation
     */
    public static Element objectToDOM(Object obj, QName qName) throws Exception {
        MessageElement messageElement = new MessageElement();
        messageElement.setQName(qName);
        messageElement.setObjectValue(obj);
        return messageElement.getAsDOM();
    }
    
}

/**
 * Creates a context for deserialization during XML de/serialization
 */
class ObjectDeserializationContext extends DeserializationContext {

    private Deserializer deserializer = null;

    public ObjectDeserializationContext(InputSource input, Class javaClass, MessageContext context) {
        super(context, new SOAPHandler());
        popElementHandler();
        QName defaultXMLType = getTypeMapping().getTypeQName(javaClass);
        this.deserializer = getDeserializerForClass(javaClass);
        pushElementHandler(
              new EnvelopeHandler((SOAPHandler) this.deserializer)
        );
        this.inputSource = input;
    }

    public Object getValue() {
        return this.deserializer.getValue();
    }

    public MessageElement getMessageElement() {
        return ((SOAPHandler)this.deserializer).myElement;
    }

    public QName getQName() {
        return getMessageElement().getQName();
    }

}
 
