/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import javax.xml.namespace.QName;
import uk.org.ogsadai.dais.xml.services.daix.XMLSerializationParameters;
import uk.org.ogsadai.dais.xml.services.daix.XMLSerializationParametersNormalizeUnicode;
import uk.org.ogsadai.dais.xml.services.daix.YesNoOmitType;
import uk.org.ogsadai.dais.xml.services.daix.YesNoType;

/**
 * Utility class to create XQuery serialization parameters used
 * by the WS-DAIX XMLSequenceAccess, XPath and XQuery port types.
 * 
 * @author Steven Lynden
 */
public class XQuerySerializationParametersUtils {

    /**
     * Construct a set of XQuery serialization parameters with the following values:
     * 
     * ByteOrderMark = yes
     * Encoding = UTF-16
     * EscapeURIAttributes = NO
     * IncludeContentType = NO
     * MediaType = MIME
     * OmitXMLDeclaration = NO
     * Version = 1.0
     * 
     * @return The default XQuery serialization parameters used by the WS-DAIX client.
     * These parameters are sent in requests when XQuery serialization parameters are not
     * specified as an input parameter. This applies to the following operations:
     * getItems (XMLSequenceAccessPT)
     * XPathExecute (XPathAccessPT)
     * XQueryExecute (XQueryAccessPT)
     */
    public static XMLSerializationParameters constructDefaultSerializationParameters() {
        XMLSerializationParameters params = new XMLSerializationParameters();
        params.setByteOrderMark( YesNoType.yes );
        params.setEncoding("UTF-16");
        params.setCdataSectionElements(new QName[0]);
        params.setEscapeUriAttributes( YesNoType.no );
        params.setIncludeContentType( YesNoType.no );
        params.setIndent( YesNoType.no );
        params.setMediaType("MIME");
        params.setNormalizeUnicode( XMLSerializationParametersNormalizeUnicode.value6 );
        params.setOmitXmlDeclaration( YesNoType.no );
        params.setStandalone( YesNoOmitType.omit );
        params.setUndeclaredNamespaces( YesNoType.yes );
        params.setUseCharacterMaps("");
        params.setVersion("1.0");
        return params;
    }
    
    
}
