/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import junit.framework.*;
import uk.org.ogsadai.dais.xml.service.core.*;
import uk.org.ogsadai.wsdaix.client.*;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the CoreDataAccess port type.
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 *
 * TODO Implement a test for GenericQuery once it is supported
 * by an implementation
 *
 */
public class CoreDataAccessTest extends TestCase {
    
    private static Logger mLog = Logger.getLogger(CoreDataAccessTest.class);
    
    /*
     * URL of the service to be tested
     */
    private String serviceURL;
    
    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;
    
    /*
     * A non-existent resource abstract name
     */
    private URI invalidResourceName;
    
    /**
     * Set up the test suite
     */
    public CoreDataAccessTest() throws Exception {
        setUp();   
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public CoreDataAccessTest(String name) {
        super(name);
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(CoreDataAccessTest.class);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        serviceURL =  p.getProperty(TestProperties.CORE_DATA_ACCESS_TEST_ENDPOINT);
        resourceName = new URI("daix:testresource");
        invalidResourceName = new URI("daix:invalidresource");
    }
    
    /**
     * Tests the GetDataResourcePropertyDocument.
     * Checks that received documents contains the correct
     * resource name and identifies the resource as being service
     * managed. Other properties are implementation dependent 
     * and cannot be tested.
     */
    public void testGetDataResourcePropertyDocument() {
        try {
            PropertyDocumentType p = ClientToolkit.getDataResourcePropertyDocument(serviceURL,resourceName);
            assertEquals( "Resource name", resourceName, p.getDataResourceAbstractName() );
            assertEquals( "Data resource management", DataResourceManagement._ExternallyManaged, p.getDataResourceManagement() );
        } catch ( Exception e ) {
            fail(e.toString());   
        }
    }
    
    /**
     * Tests the GetDataResourcePropertyDocument
     * Attemps to execute the operation specifying a resource name that doesn't
     * exist. Checks that an InvalidResourceNameFault is generated.
     */
    public void testGetDataResourcePropertyDocument_InvalidResourceNameFault() {
        try {
            ClientToolkit.getDataResourcePropertyDocument(serviceURL,invalidResourceName);
            String error = "Service accepted invalid resource name"; 
            fail(error);
        } catch ( InvalidResourceNameFaultType f ) {
            assertTrue("Invalid resource name fault",true);
        } catch ( Exception e ) {
            fail(e.toString());   
        }
    }
    
    /**
     * Tests the Destroy operation.
     * Attempts to destroy the resource. This should fail because the
     * resource is externally managed. Checks that a NotAuthorizedFault
     * is generated.
     */
    public void testDestroyDataResource_NotAuthorizedFault() {
        try {
            ClientToolkit.destroyDataResource(serviceURL,resourceName);
            String error = "Service accepted request to destroy externally managed resource";
        } catch ( NotAuthorizedFaultType f ) {
            assertTrue("Not authorized fault",true);   
        } catch ( Exception e ) {
            fail(e.toString());
        }
    }
    
    public void tearDown() {
    
    }
    
    
}
