/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test;

import junit.framework.*;
import uk.org.ogsadai.dais.xml.service.core.*;
import uk.org.ogsadai.wsdaix.client.*;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import uk.org.ogsadai.wsdaix.test.properties.TestProperties;

/**
 * @author Steven Lynden
 *
 * Contains test cases for the CoreResourceList port type.
 * 
 * For instructions on how to execute the tests please see
 * the doc/UserGuide.doc 
 *
 */
public class CoreResourceListTest extends TestCase {

    private static Logger mLog = Logger.getLogger(CoreResourceListTest.class);
    
    /*
     * URL of the service to be tested
     */
    private String serviceURL;
    
    /*
     * Abstract name of the resource to be tested 
     */
    private URI resourceName;
    
    /**
     * Set up the test suite
     */
    public CoreResourceListTest() throws Exception {
        setUp();   
    }
    
    /**
     * Set up the test suite
     * @param name Name given to the test suite
     */
    public CoreResourceListTest(String name) {
        super(name);   
    }
    
    /**
     * Main method used for executing the tests
     * @deprecated May not execute tests in the correct order
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(CoreResourceListTest.class);
    }
    
    /**
     * Set up variables used during testing 
     */
    public void setUp() throws Exception {
        TestProperties p = new TestProperties();
        serviceURL =  p.getProperty(TestProperties.CORE_RESOURCE_LIST_TEST_ENDPOINT);
        resourceName = new URI("daix:testresource");
    }
    
    public void tearDown() {
    
    }
    
    /*
     * Tests the GetResourceList operation.
     * Checks that the resource list contains on resource with the correct
     * resource name.
     */
    public void testGetResourceList() {
        try {
            DataResourceAddressType[] addr = ClientToolkit.getResourceList(serviceURL);
            assertEquals("Number of exposed resources",1,addr.length);
            assertEquals("Resource name",resourceName.toString(),addr[0].getReferenceParameters().get_any()[0].getFirstChild().getNodeValue() );
        } catch ( Exception e ) {
            mLog.error("Exception: " + e);
            fail(e.toString());
        }
    }
    
    /*
     * Tests the Resolve operation.
     * Checks that the endpoint used to access the externally managed resource
     * is returned in response to the Resolve operation.
     */
    public void testResolve() {
        try {
            boolean addrFound = false;
            DataResourceAddressType[] addr = ClientToolkit.resolve(serviceURL,resourceName);
            for(int i=0;i<addr.length;i++) {
                if ( addr[i].getAddress().toString().equals(serviceURL) ) 
                    addrFound = true;
            }
            assertEquals("Address " + serviceURL + " found in resolve response ",true,addrFound);
        } catch ( Exception e ) {
            mLog.error("Exception: " + e);
            fail(e.toString());
        }
    }
    
    
}
