/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test.properties;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * @author Steven Lynden
 * 
 * Properties used by the tests, some are loaded from 
 * client/systemtest/test.properties
 * 
 * This file must be edited to contain the endpoints used
 * during testing and properties of the implementation tested.
 * 
 * Ensure that the following parameter is correctly set in the 
 * properties document: 
 * 
 * - If the resource supported nested collections, the number of
 * documents expected should 5.
 * 
 * - If the resource does not support nested collections, the
 * number of documents expected should be 1.
 *
 */
public class TestProperties {

    /*
     * Properties object loaded from client/systemtest/test.properties
     */
    private Properties properties;
    
    /**
     * Endpoint used to test the CoreResourceList port type 
     */
    public static String CORE_RESOURCE_LIST_TEST_ENDPOINT = "CoreResourceListTest.url";
    
    /**
     * Endpoint used to test the CoreDataAccess port type 
     */
    public static String CORE_DATA_ACCESS_TEST_ENDPOINT = "CoreDataAccessTest.url";
    
    /**
     * Endpoint used to test the CollectionAccess port type 
     */
    public static String COLLECTION_ACCESS_TEST_ENDPOINT =  "CollectionAccessTest.url";
    
    /**
     * Endpoint used to test the CollectionFactory port type 
     */
    public static String COLLECTION_FACTORY_TEST_ENDPOINT =  "CollectionFactoryTest.url";
    
    /**
     * Endpoint used to test the CoreDataAccess port type from within the CollectionFactory
     * tests (this endpoint is used to test successfully destroying a resource) 
     */
    public static String COLLECTION_FACTORY_TEST_CORE_DATA_ACCESS_ENDPOINT = "CollectionFactoryTest.coreDataAccess.url";
    
    /**
     * Endpoint used to test the XQueryAccess port type 
     */
    public static String XQUERY_ACCESS_TEST_ENDPOINT =  "XQueryAccessTest.url";
    
    /**
     * Endpoint used to test the XQueryFactory port type 
     */
    public static String XQUERY_FACTORY_TEST_ENDPOINT =  "XQueryFactoryTest.url";
    
    /**
     * Endpoint used to test the XPathAccess port type 
     */
    public static String XPATH_ACCESS_TEST_ENDPOINT =  "XPathAccessTest.url";
    
    /**
     * Endpoint used to test the XPathFactory port type 
     */
    public static String XPATH_FACTORY_TEST_ENDPOINT =  "XPathFactoryTest.url";
    
    /**
     * Endpoint used to test the XUpdate port type 
     */
    public static String XUPDATE_TEST_ENDPOINT =  "XUpdateTest.url";
    
    /**
     * The number of documents that should be contained in the database. 
     */
    public static String DATABASE_NUM_DOCUMENTS = "database.documents";
    
    public TestProperties() throws IOException {
        this("systemtest/test.properties");
    }
    
    /**
     * Initialised the properties by loading from the specified
     * file.
     */
    public TestProperties(String filename) throws IOException {
        properties = new Properties();
        properties.load( new FileInputStream(filename));
    }
    
    /**
     * Get a property from a test properties object
     * @param name the name of the property
     */
    public String getProperty(String name) {
        return properties.getProperty(name).trim();
    }
    
}
