/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.test.util;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.modules.CollectionManagementService;
import org.xmldb.api.modules.XMLResource;
import uk.org.ogsadai.common.xml.XMLUtilities;
import org.w3c.dom.*;

/**
 * @author Steven Lynden
 *
 * Load the databases with the contents needed to
 * run the tests. Assumes the the resources/database
 * support nested collections. This class must be
 * modified if this is not the case.
 * 
 * Works for the Exist XML database only.
 *  
 */
public class CreateTestDatabase {

    /**
     * Populates the test database using the default
     * URL and port number:
     * - localhost, 8080 
     */
    public static void main(String[] args) {
        try {
            create();
        } catch ( Exception e ) { 
            System.out.println(e);
            e.printStackTrace();
        }
    }
        
    /**
     * Populates the test database using the default
     * URL and port number:
     * - localhost, 8080 
     */
    public static void create() throws Exception {
        create("org.exist.xmldb.DatabaseImpl","localhost",8080);   
    }
    
    /**
     * Populate the test database
     * @param driver database driver
     * @param host host on which the database is running
     * @param portNumber the port number of the database server
     */
    public static void create(String driver, String host, int portNumber) throws Exception {
        String port = Integer.toString(portNumber);
        Class c = Class.forName( driver );
        Database db = (Database) c.newInstance();
        DatabaseManager.registerDatabase( db );
        String database = "db";
        String existURL = "xmldb:exist://" + host + ":" + port + "/exist/xmlrpc/" + database + "/";
        Collection collection = DatabaseManager.getCollection( existURL );
        CollectionManagementService service = (CollectionManagementService) collection.getService("CollectionManagementService", "1.0");
        String newCollectionName = "contacts";
        Collection child = collection.getChildCollection(newCollectionName);
        child = service.createCollection(newCollectionName);
        //create subcollections
        CollectionManagementService childServ = (CollectionManagementService) child.getService("CollectionManagementService", "1.0");
        Collection epcc = childServ.createCollection("epcc");
        Collection ibm = childServ.createCollection("ibm");
        CollectionManagementService ibmService = (CollectionManagementService) ibm.getService("CollectionManagementService", "1.0");
        Collection hursley = ibmService.createCollection("hursley");
        //add the documents        
        addDocument( "systemtest/antonioletti.xml", hursley );
        addDocument( "systemtest/atkinson.xml", hursley );
        addDocument( "systemtest/borley.xml", epcc );
        addDocument( "systemtest/hume.xml", epcc );
        addDocument( "systemtest/sugden.xml", child );
        //close everything
        epcc.close();
        hursley.close();
        ibm.close();
        child.close();
        collection.close();
    }
    
    /*
     * Add the specified document to the database collection
     */
    private static void addDocument( String documentFileName, Collection collection ) throws Exception {
        String documentString, resourceID;
        XMLResource document;
        Document xmlDocument;
        xmlDocument = XMLUtilities.xmlFileToDOM(documentFileName,false);
        resourceID = ((Element)xmlDocument.getElementsByTagName("entry").item(0)).getAttribute("id");
        documentString = XMLUtilities.xmlDOMToString( xmlDocument );
        document = (XMLResource) collection.createResource(resourceID, "XMLResource");
        document.setContent(documentString);
        collection.storeResource( document );
    }
    
}
