/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.common;

import javax.xml.namespace.QName;
import org.apache.axis.types.URI;

/**
 * @author Steven Lynden
 *
 * Stores the values of various static properties and constants
 * of WS-DAIX. 
 *
 */
public class DAIXProperties {

    /**
     * The QName used for XML results generated by WS-DAIX services
     */
    public static final QName OGSA_DAI_RESULTS_QNAME = new QName("http://ogsadai.org.uk","results");
    
    /**
     * XQueryX namespace
     */
    public static final String XQUERYX_NS = "http://www.w3.org/2005/XQueryX";
    
    /**
     * XUpdate namespace
     */
    public static final String XUPDATE_NS = "http://www.xmldb.org/xupdate";
    
    /**
     * XUpdate URI
     */
    public static URI XUpdateURI;
    
    /**
     * XQueryX URI
     */
    public static URI XQueryXURI;
    
    /**
     * The default dataset format URI, used to indicate the format of XML results
     * generated by the WS-DAIX services. The default is "ogsadai:resourceset", which
     * formats results in the same way as OGSA-DAI XML data resources.
     */
    public static URI defaultDataSetFormatURI;
    
    /**
     * WS-DAI namespace
     */
    public static final String WS_DAI_NS = "http://www.ggf.org/namespaces/2005/12/WS-DAI";
    
    /**
     * WS-DAIX namespace
     */
    public static final String WS_DAIX_NS = "http://www.ggf.org/namespaces/2005/12/WS-DAIX";
    
    /**
     * WS-Addressing namespace
     */
    public static final String WS_ADDR_NS = "http://www.w3.org/2005/08/addressing";
    
    /**
     * Local filename of the WS-DAIX deployment properties file
     */
    public static final String DEPLOYMENT_PROPERTIES_FILE = "wsdaix-deployment.properties";
    
    /**
     * Constant indicating that a service is an XML Collection service
     */
    public static final String COLLECTION_SERVICE = "CollectionService";
    
    /**
     * Constant indicating that a service is an XML Document service
     */
    public static final String DOCUMENT_SERVICE = "DocumentService";
    
    /**
     * Constant indicating that a service is an XML Sequence service
     */
    public static final String SEQUENCE_SERVICE = "SequenceService";
    
    /*
     * Static code to initialise the dataset format, XQueryX and XUpdate
     * URIs on startup.
     */
    static {
        try {
            defaultDataSetFormatURI = new URI("ogsadai:resourceset");
            XQueryXURI = new URI(XQUERYX_NS);
            XUpdateURI = new URI(XUPDATE_NS);
        } catch ( Exception  e ) {
            
        }
    }
    
    /**
     * Messages
     */
    public static final QName XQueryExecuteFactory = new QName(WS_DAIX_NS,"XQueryExecuteFactory"); 
    public static final QName XQueryExecute = new QName(WS_DAIX_NS,"XQueryExecute");
    public static final QName XUpdateExecute = new QName(WS_DAIX_NS,"XUpdateExecute");
    public static final QName XPathExecuteFactory = new QName(WS_DAIX_NS,"XPathExecuteFactory"); 
    public static final QName XPathExecute = new QName(WS_DAIX_NS,"XPathExecute");
    public static final QName GetItems = new QName(WS_DAIX_NS,"GetItems");
    public static final QName GetXMLSequencePropertyDocument = new QName(WS_DAIX_NS,"GetXMLSequencePropertyDocument");
    public static final QName DocumentSelectionFactory = new QName(WS_DAIX_NS,"DocumentSelectionFactory");
    public static final QName CollectionSelectionFactory = new QName(WS_DAIX_NS,"CollectionSelectionFactory");
    public static final QName RemoveSchema = new QName(WS_DAIX_NS,"RemoveSchema");
    public static final QName AddSchema = new QName(WS_DAIX_NS,"AddSchema");
    public static final QName RemoveSubcollection = new QName(WS_DAIX_NS,"RemoveSubcollection");
    public static final QName CreateSubcollection = new QName(WS_DAIX_NS,"CreateSubcollection");
    public static final QName AddDocuments = new QName(WS_DAIX_NS,"AddDocuments");
    public static final QName RemoveDocuments = new QName(WS_DAIX_NS,"RemoveDocuments");
    public static final QName GetXMLCollectionPropertyDocument = new QName(WS_DAIX_NS,"GetCollectionPropertyDocument");
    public static final QName Resolve = new QName(WS_DAI_NS,"Resolve");
    public static final QName GetResourceList = new QName(WS_DAI_NS,"GetResourceList");
    public static final QName GenericQuery = new QName(WS_DAI_NS,"GenericQuery");
    public static final QName DestroyDataResource = new QName(WS_DAI_NS,"DestroyDataResource");
    public static final QName GetDataResourcePropertyDocument = new QName(WS_DAI_NS,"GetDataResourcePropertyDocument");
    
    /**
     * Types
     */
    public static final QName ConfigurationDocumentType = new QName(WS_DAI_NS,"ConfigurationDocumentType");
    public static final QName XMLCollectionConfigurationDocumentType = new QName(WS_DAIX_NS,"XMLCollectionConfigurationDocumentType");
    public static final QName CollectionType = new QName(WS_DAIX_NS,"CollectionType");
    
    /**
     * PortTypes
     */
    public static final QName XMLSequenceAccessPT = new QName(WS_DAIX_NS,"XMLSequenceAccessPT");
    public static final QName XMLCollectionAccessPT = new QName(WS_DAIX_NS,"XMLCollectionAccessPT");
    public static final QName XMLCollectionFactoryPT = new QName(WS_DAIX_NS,"XMLCollectionFactoryPT");
    public static final QName XQueryAccessPT = new QName(WS_DAIX_NS,"XQueryAccessPT");
    public static final QName XQueryFactoryPT = new QName(WS_DAIX_NS,"XQueryFactoryPT");
    public static final QName XPathAccessPT = new QName(WS_DAIX_NS,"XPathAccessPT");
    public static final QName XPathFactoryPT = new QName(WS_DAIX_NS,"XPathFactoryPT");
    public static final QName CoreDataAccessPT = new QName(WS_DAI_NS,"CoreDataAccessPT");
    public static final QName CoreResourceListPT = new QName(WS_DAI_NS,"CoreResourceListPT");
    public static final QName XUpdateAccessPT = new QName(WS_DAIX_NS,"XUpdateAccessPT");
    
}
