/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.core;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.properties.PropertyValueException;
import uk.org.ogsadai.dais.xml.service.core.BaseRequestType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceDescription;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.DatasetDataType;
import uk.org.ogsadai.dais.xml.service.core.DatasetType;
import uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceRequest;
import uk.org.ogsadai.dais.xml.service.core.DestroyDataResourceResponse;
import uk.org.ogsadai.dais.xml.service.core.GenericQueryRequest;
import uk.org.ogsadai.dais.xml.service.core.GenericQueryResponse;
import uk.org.ogsadai.dais.xml.service.core.GetDataResourcePropertyDocumentRequest;
import uk.org.ogsadai.dais.xml.service.core.GetResourceListRequest;
import uk.org.ogsadai.dais.xml.service.core.GetResourceListResponse;
import uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.service.core.ResolveRequest;
import uk.org.ogsadai.dais.xml.service.core.ResolveResponse;
import uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.AddDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.AddSchemaRequest;
import uk.org.ogsadai.dais.xml.services.daix.AddSchemaResponse;
import uk.org.ogsadai.dais.xml.services.daix.CollectionAlreadyExistsFaultType;
import uk.org.ogsadai.dais.xml.services.daix.CollectionSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.CreateSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.DocumentDoesNotExistFaultType;
import uk.org.ogsadai.dais.xml.services.daix.DocumentSelectionFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetItemsRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetItemsResponse;
import uk.org.ogsadai.dais.xml.services.daix.GetSchemaRequest;
import uk.org.ogsadai.dais.xml.services.daix.GetSchemaResponse;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCollectionNameFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidCountFaultType;
import uk.org.ogsadai.dais.xml.services.daix.InvalidStartPositionFaultType;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveDocumentsResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSchemaResponse;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionRequest;
import uk.org.ogsadai.dais.xml.services.daix.RemoveSubcollectionResponse;
import uk.org.ogsadai.dais.xml.services.daix.SchemaAdditionMakesDocumentsInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaDoesNotExistFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.SchemaRemovalMakesDocumentsInvalidFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XMLSequencePropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XPathFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XQueryFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateFaultType;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.wsdaix.common.CollectionAccessor;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;
import uk.org.ogsadai.wsdaix.porttypes.CorePT;
import uk.org.ogsadai.wsdaix.porttypes.XMLCollectionAccessImpl;
import uk.org.ogsadai.wsdaix.porttypes.XMLCollectionFactoryImpl;
import uk.org.ogsadai.wsdaix.porttypes.XPathXQueryXUpdatePT;
import uk.org.ogsadai.wsdaix.resource.DAIXDataResourceAccessor;
import uk.org.ogsadai.wsdaix.resource.TransientXMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLDocumentResource;
import uk.org.ogsadai.wsdaix.resource.XMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.XMLSequenceResource;

/**
 * 
 * Initialises WS-DAIX and delegates operations to port type
 * implementation classes.
 * 
 * @author Steven Lynden
 *
 */
public class DAIXImpl {

    private CorePT coreDataAccess;
    private XPathXQueryXUpdatePT queryPT;
    private XMLCollectionFactoryImpl xmlCollectionFactory;
    private XMLCollectionAccessImpl xmlCollectionAccess;
    private Map dataServiceResources;
    private DeploymentProperties deploymentProperties;
    private DataServiceResourceFactory dataServiceResourceFactory;
    private String configDir;
    
    private static Logger mLog = Logger.getLogger(DAIXImpl.class);
    
    /**
     * Constructs a new instance of this class and loads properties from the deployment properties
     * file.
     */
    public DAIXImpl(Map dataServiceResources, String configDir, DataServiceResourceFactory dataServiceResourceFactory, String url) {
        this.dataServiceResources = dataServiceResources;
        this.configDir = configDir;
        try {
            this.deploymentProperties = new DeploymentProperties(configDir+"/"+DAIXProperties.DEPLOYMENT_PROPERTIES_FILE,url);
            mLog.debug("Deployment properties loaded from wsdaix-deployment.properties");
        } catch ( Exception e) {
            mLog.error("FATAL ERROR: Deployment properties file WAS NOT loaded");   
        }
        /*
         * Ensure that all data service resources have access to deployment properties
         * and initialise port type implementations
         */
        assignDeploymentProperties();
        coreDataAccess = new CorePT(dataServiceResourceFactory,deploymentProperties);
        queryPT = new XPathXQueryXUpdatePT(dataServiceResourceFactory,deploymentProperties);
        /*  
         * initialise the mapping between OGSA-DAI and WS-DAIX collections
         */
        Iterator it = this.dataServiceResources.values().iterator();
        /* initially the data service resources map contains one resource */
        DataServiceResource dsr = (DataServiceResource) it.next();
        XMLCollectionResource dra = (XMLCollectionResource) dsr.getDataResourceAccessor();
        Property collectionSchemaProperty = dra.getCollectionSchemaProperty();
        try {
            CollectionAccessor collectionAccessor =
                new CollectionAccessor(collectionSchemaProperty);
            dra.setCollectionAccessor(collectionAccessor);
            /*
             * Collection portTypes require a reference to the collection
             * accessor object
             */
            xmlCollectionAccess = new XMLCollectionAccessImpl(
                    dataServiceResourceFactory,
                    deploymentProperties,
                    collectionAccessor);
            xmlCollectionFactory = new XMLCollectionFactoryImpl(
                    dataServiceResourceFactory,
                    deploymentProperties,
                    collectionAccessor);
        } catch ( PropertyValueException e ) {
            mLog.error("Error initialising collection accessor: " + e.toString() );
        }
        mLog.debug("WS-DAIX initialised: " + url + " loc: " + this.configDir);
    }

    /**
     * Implements CoreDataAccess::GetDataResourcePropertyDocument
     */
    public PropertyDocumentType getDataResourcePropertyDocument(GetDataResourcePropertyDocumentRequest getDataResourcePropertyDocumentRequest) throws RemoteException, InvalidResourceNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        DataResourceAccessor resource = getDataResourceAccessor(getDataResourcePropertyDocumentRequest);
        PropertyDocumentType propertyDoc = ((DAIXDataResourceAccessor)resource).getDataResourcePropertyDocument();
        DataServiceResource dsr = getDataServiceResource(getDataResourcePropertyDocumentRequest);
        assignResourceAbstractName(propertyDoc,dsr);
        assignDataResourceDescription(propertyDoc);
        return propertyDoc; 
    }
    
    /**
     * Implements XMLSequenceAccess::GetXMLSequencePropertyDocument
     */
    public XMLSequencePropertyDocumentType getXMLSequencePropertyDocument(GetDataResourcePropertyDocumentRequest getXMLSequencePropertyDocumentRequest) throws RemoteException, InvalidResourceNameFaultType, DataResourceUnavailableFaultType  {
        DataResourceAccessor resource = getDataResourceAccessor(getXMLSequencePropertyDocumentRequest);
        if ( ! ( resource instanceof XMLSequenceResource ) ) throw new InvalidResourceNameFaultType();
        XMLSequencePropertyDocumentType propertyDoc = ((XMLSequenceResource)resource).getXMLSequencePropertyDocument();
        DataServiceResource dsr = getDataServiceResource(getXMLSequencePropertyDocumentRequest);
        assignResourceAbstractName(propertyDoc,dsr);
        assignDataResourceDescription(propertyDoc);
        return propertyDoc; 
    }
    
    /**
     * Implements XMLCollectionAccess::GetCollectionPropertyDocument
     */
    public XMLCollectionPropertyDocumentType getCollectionPropertyDocument(GetDataResourcePropertyDocumentRequest getCollectionPropertyDocumentRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        XMLCollectionPropertyDocumentType propertyDoc = null;
        DataResourceAccessor resource = getDataResourceAccessor(getCollectionPropertyDocumentRequest);
        if ( resource instanceof XMLCollectionResource ) {
            propertyDoc = ((XMLCollectionResource) resource).getCollectionPropertyDocument();
        } else if ( resource instanceof TransientXMLCollectionResource ) {
            propertyDoc = ((TransientXMLCollectionResource) resource).getCollectionPropertyDocument();
        } 
        DataServiceResource dsr = getDataServiceResource(getCollectionPropertyDocumentRequest);
        assignResourceAbstractName(propertyDoc,dsr);
        assignDataResourceDescription(propertyDoc);
        return propertyDoc;        
    }
    
    /*
     * Assigns the data resource abstract name to a property document
     */
    private static void assignResourceAbstractName(PropertyDocumentType propertyDocument, DataServiceResource dataServiceResource) throws InvalidResourceNameFaultType {
        try {
            propertyDocument.setDataResourceAbstractName(new URI("daix",dataServiceResource.getID().toString()));
        } catch ( Exception e ) {
            throw new InvalidResourceNameFaultType();   
        }
    }
    
    /*
     * Fills into the property document a basic description of WS-DAIX data resources
     */
    private static void assignDataResourceDescription(PropertyDocumentType propertyDocument) {
        DataResourceDescription desc = new DataResourceDescription();
        try {
            MessageElement e = new MessageElement(DAIXProperties.WS_DAI_NS,"DataResource");
            e.setObjectValue("OGSA-DAI WS-DAIX resource");
            desc.set_any(new MessageElement[] {e} );
        } catch ( Exception e ) { }
        propertyDocument.setDataResourceDescription(desc);
    }
    
    /**
     * Implements CoreDataAccess::DestroyDataResource
     */
    public DestroyDataResourceResponse destroyDataResource(DestroyDataResourceRequest destroyDataResourceRequest) throws RemoteException, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        //can't destroy externally managed resources
        DataResourceAccessor resource = getDataResourceAccessor(destroyDataResourceRequest);
        if ( resource instanceof XMLCollectionResource ) {
            throw new NotAuthorizedFaultType();
        }
        String id = destroyDataResourceRequest.getDataResourceAbstractName().getPath(false,true);
        synchronized(dataServiceResources) {
            if (dataServiceResources.containsKey(id))
                dataServiceResources.remove(id);            
            else {
                throw new InvalidResourceNameFaultType();
            }
        }
        return new DestroyDataResourceResponse();
    }
    
    /**
     * CoreDataAccess::GenericQuery - this operation is currently not implemented
     */
    public GenericQueryResponse genericQuery(GenericQueryRequest genericQueryRequest) throws RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        return coreDataAccess.genericQuery(genericQueryRequest);
    }
    
    /**
     * Implements CoreResourceList::GetResourceList for collection services only
     */
    public GetResourceListResponse getCollectionServiceResourceList(GetResourceListRequest getResourceListRequest) throws RemoteException, NotAuthorizedFaultType {
        String url = deploymentProperties.getProperty(DeploymentProperties.COLLECTION_CORE_RESOURCE_LIST_ENDPOINT);
        GetResourceListResponse response = new GetResourceListResponse();
        Vector addr = new Vector();
        synchronized(this.dataServiceResources) {
            Iterator i = this.dataServiceResources.keySet().iterator();
            while(i.hasNext()) {
                String resourceID = (String) i.next();
                if ( (((DataServiceResource) dataServiceResources.get(resourceID)).getDataResourceAccessor() instanceof XMLCollectionResource ) || (((DataServiceResource) dataServiceResources.get(resourceID)).getDataResourceAccessor() instanceof TransientXMLCollectionResource) )
                    addr.add( DAIXUtil.constructDataResourceAddressType(url,resourceID) );
            }
        }
        response.setDataResourceAddress( (DataResourceAddressType[]) addr.toArray(new DataResourceAddressType[addr.size()]));  
        return response;
    }
    
    /**
     * Implements CoreResourceList::GetResourceList for document services only
     */
    public GetResourceListResponse getDocumentServiceResourceList(GetResourceListRequest getResourceListRequest) throws RemoteException, NotAuthorizedFaultType {
        String url = deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_CORE_RESOURCE_LIST_ENDPOINT);
        GetResourceListResponse response = new GetResourceListResponse();
        Vector addr = new Vector();
        synchronized(this.dataServiceResources) {
            Iterator i = this.dataServiceResources.keySet().iterator();
            while(i.hasNext()) {
                String resourceID = (String) i.next();
                if ( ((DataServiceResource) dataServiceResources.get(resourceID)).getDataResourceAccessor() instanceof TransientXMLDocumentResource )
                    addr.add( DAIXUtil.constructDataResourceAddressType(url,resourceID) );
            }
        }
        response.setDataResourceAddress( (DataResourceAddressType[]) addr.toArray(new DataResourceAddressType[addr.size()]));        
        return response;
    }
    
    /**
     * Implements CoreResourceList::GetResourceList for sequence services only
     */
    public GetResourceListResponse getSequenceServiceResourceList(GetResourceListRequest getResourceListRequest) throws RemoteException, NotAuthorizedFaultType {
        String url = deploymentProperties.getProperty(DeploymentProperties.SEQUENCE_CORE_RESOURCE_LIST_ENDPOINT);
        GetResourceListResponse response = new GetResourceListResponse();
        Vector addr = new Vector();
        synchronized(this.dataServiceResources) {
            Iterator i = this.dataServiceResources.keySet().iterator();
            while(i.hasNext()) {
                String resourceID = (String) i.next();
                if ( ((DataServiceResource) dataServiceResources.get(resourceID)).getDataResourceAccessor() instanceof XMLSequenceResource )
                    addr.add( DAIXUtil.constructDataResourceAddressType(url,resourceID) );
            }
        }
        response.setDataResourceAddress( (DataResourceAddressType[]) addr.toArray(new DataResourceAddressType[addr.size()]));        
        return response;   
    }
    
    /**
     * Implements CoreResourceList::Resolve
     * Given a resource abstract name, returns all endpoints exposing the resource. 
     */
    public ResolveResponse resolve(ResolveRequest resolveRequest) throws RemoteException, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        ResolveResponse response = new ResolveResponse();
        DataResourceAddressType[] addr = null; 
        DataResourceAccessor dsr = this.getDataResourceAccessor(resolveRequest);
        if ( (dsr instanceof XMLCollectionResource) || ( dsr instanceof TransientXMLCollectionResource ) ) {
            addr = new DataResourceAddressType[9];
            addr[0] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_COLLECTION_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[1] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_COLLECTION_FACTORY_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[2] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_CORE_DATA_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[3] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_CORE_RESOURCE_LIST_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[4] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XPATH_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[5] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XPATH_FACTORY_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[6] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XQUERY_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[7] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XQUERY_FACTORY_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[8] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XUPDATE_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
        } else if ( dsr instanceof XMLSequenceResource ) {
            addr = new DataResourceAddressType[3];
            addr[0] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.SEQUENCE_CORE_DATA_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[1] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.SEQUENCE_CORE_RESOURCE_LIST_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[2] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.SEQUENCE_SEQUENCE_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
        } else if ( dsr instanceof TransientXMLDocumentResource ) {
            addr = new DataResourceAddressType[7];
            addr[0] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_CORE_DATA_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[1] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_CORE_RESOURCE_LIST_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[2] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XPATH_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[3] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XPATH_FACTORY_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[4] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XQUERY_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[5] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XQUERY_FACTORY_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
            addr[6] = DAIXUtil.constructDataResourceAddressType(
                    deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XUPDATE_ACCESS_ENDPOINT),
                    resolveRequest.getDataResourceAbstractName());
        }
        response.setDataResourceAddress(addr);
        return response;   
    }
    
    /**
     * Implements CollectionAccess::GetDocuments
     */
    public uk.org.ogsadai.dais.xml.services.daix.GetDocumentsResponse getDocuments(GetDocumentsRequest getDocumentsRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(getDocumentsRequest);
        return xmlCollectionAccess.getDocuments(getDocumentsRequest,resource);    
    }
    
    /**
     * Implements CollectionAccess::AddDocuments
     */
    public AddDocumentsResponse addDocuments(AddDocumentsRequest addDocumentsRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(addDocumentsRequest);
        return xmlCollectionAccess.addDocuments(addDocumentsRequest,resource);
    }
    
    /**
     * Implements CollectionAccess::RemoveDocuments
     */
    public RemoveDocumentsResponse removeDocuments(RemoveDocumentsRequest removeDocumentsRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(removeDocumentsRequest);
        return xmlCollectionAccess.removeDocuments(removeDocumentsRequest,resource);   
    }
    
    /**
     * Implements CollectionAccess::CreateSubcollection
     */
    public CreateSubcollectionResponse createSubcollection(CreateSubcollectionRequest createSubcollectionRequest) throws RemoteException, CollectionAlreadyExistsFaultType, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(createSubcollectionRequest);
        return xmlCollectionAccess.createSubcollection(createSubcollectionRequest,resource);
    }
    
    /**
     * Implements CollectionAccess::RemoveSubcollection
     */
    public RemoveSubcollectionResponse removeSubcollection(RemoveSubcollectionRequest removeSubcollectionRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(removeSubcollectionRequest);
        return xmlCollectionAccess.removeSubcollection(removeSubcollectionRequest,resource);    
    }
    
    /**
     * Implements CollectionAccess::AddSchema
     * Note: currently not implemented
     */
    public AddSchemaResponse addSchema(AddSchemaRequest addSchemaRequest) throws RemoteException, SchemaInvalidFaultType, ServiceBusyFaultType, SchemaAdditionMakesDocumentsInvalidFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        mLog.error("AddSchema is not supported");
        throw new NotAuthorizedFaultType();    
    }
    
    /**
     * Implements CollectionAccess::GetSchema
     * Note: currently not implemented
     */
    public GetSchemaResponse getSchema(GetSchemaRequest getSchemaRequest) throws NotAuthorizedFaultType {
        mLog.error("GetSchema is not supported");
        throw new NotAuthorizedFaultType();    
    }
    
    /**
     * Implements CollectionAccess::RemoveSchema
     * Note: currently not implemented
     */
    public RemoveSchemaResponse removeSchema(RemoveSchemaRequest removeSchemaRequest) throws RemoteException, SchemaRemovalMakesDocumentsInvalidFaultType, ServiceBusyFaultType, SchemaDoesNotExistFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        mLog.error("RemoveSchema is not supported");
        throw new NotAuthorizedFaultType();   
    }
    
    /**
     * Implements CollectionFactory::CollectionSelectionFactory
     */
    public DataResourceAddressType collectionSelectionFactory(CollectionSelectionFactoryRequest collectionSelectionFactoryRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, InvalidConfigurationDocumentFaultType, InvalidPortTypeQNameFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource resource = getDataServiceResource(collectionSelectionFactoryRequest);
        return this.xmlCollectionFactory.collectionSelectionFactory(collectionSelectionFactoryRequest,resource);   
    }
    
    /**
     * Implements CollectionFactory::DocumentSelectionFactory
     */
    public DataResourceAddressType documentSelectionFactory(DocumentSelectionFactoryRequest documentSelectionFactoryRequest) throws RemoteException, ServiceBusyFaultType, InvalidResourceNameFaultType, DocumentDoesNotExistFaultType, InvalidCollectionNameFaultType, NotAuthorizedFaultType, InvalidConfigurationDocumentFaultType, InvalidPortTypeQNameFaultType, DataResourceUnavailableFaultType  {
        mLog.debug("in document selection factory");
        DataServiceResource resource = getDataServiceResource(documentSelectionFactoryRequest);
        return this.xmlCollectionFactory.documentSelectionFactory(documentSelectionFactoryRequest,resource);    
    }
    
    /**
     * Implements XMLSequenceAccess::GetItems
     */
    public GetItemsResponse getItems(GetItemsRequest getItemsRequest) throws RemoteException, InvalidStartPositionFaultType, ServiceBusyFaultType, InvalidCountFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, DataResourceUnavailableFaultType  {
        DataResourceAccessor r = getDataResourceAccessor(getItemsRequest);
        if ( ! (r instanceof XMLSequenceResource) ) throw new InvalidResourceNameFaultType();
        //validate dataset format uri if specified
        if ( getItemsRequest.getDatasetFormatURI() != null ) {
            if ( ! getItemsRequest.getDatasetFormatURI().equals(DAIXProperties.defaultDataSetFormatURI) )
                throw new InvalidDatasetFormatFaultType();
        }
        GetItemsResponse response = new GetItemsResponse();
        Document items = ((XMLSequenceResource)r).getItems(getItemsRequest.getStartPosition().intValue(),getItemsRequest.getCount().intValue());
        DatasetType data = new DatasetType();
        DatasetDataType results = new DatasetDataType();
        data.setDatasetFormatURI( DAIXProperties.defaultDataSetFormatURI );
        MessageElement resultsElem = DAIXUtil.getResultsAsMessageElement(DAIXProperties.defaultDataSetFormatURI,items);
        results.set_any(new MessageElement[] { new MessageElement(items.getDocumentElement()) });
        data.setDatasetData(results);
        response.setDataset(data);
        return response;
    }
    
    /**
     * Implements XPathAccess::XPathExecute
     */
    public XPathExecuteResponse XPathExecute(XPathExecuteRequest XPathExecuteRequest) throws RemoteException, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        mLog.debug("in XPath execute");
        DataServiceResource r = getDataServiceResource(XPathExecuteRequest);
        return queryPT.XPathExecute(XPathExecuteRequest, r);
    }
    
    /**
     * Implements XPathFactory::XPathExecuteFactory
     */
    public DataResourceAddressType XPathExecuteFactory(XPathExecuteFactoryRequest XPathExecuteFactoryRequest) throws RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType  {
        DataServiceResource r = getDataServiceResource(XPathExecuteFactoryRequest);
        return queryPT.XPathExecuteFactory(XPathExecuteFactoryRequest,r);
    }
    
    /*
     * Retrieves a data service resource using the abstract name
     * contained within a request message.
     */
    private DataServiceResource getDataServiceResource(BaseRequestType request) throws InvalidResourceNameFaultType{
        String id = request.getDataResourceAbstractName().getPath(false,true);
        mLog.debug("mapping abstract name: " + request.getDataResourceAbstractName().toString() + " to resource ID: " + id);
        return getDataServiceResource(id);
    }
    
    /*
     * Retrieves a data service resource from a string representation
     * of a data resource abstract name.
     */
    private DataServiceResource getDataServiceResource(String id) throws InvalidResourceNameFaultType{
        synchronized ( this.dataServiceResources ) {
            if (! this.dataServiceResources.containsKey(id) ) throw new InvalidResourceNameFaultType();
            return (DataServiceResource) this.dataServiceResources.get(id);
        }
    }
    
    /*
     * Retrieves a data service resource using the abstract name
     * contained within a resolve request.
     */
    private DataServiceResource getDataServiceResource(ResolveRequest request) throws InvalidResourceNameFaultType {
        String id = request.getDataResourceAbstractName().getPath(false,true);
        mLog.debug("mapping abstract name: " + request.getDataResourceAbstractName().toString() + " to resource ID: " + id);
        return getDataServiceResource(id);
    }
    
    /*
     * Assigns deployment properties to all externally managed
     * data resource accessors
     */
    private void assignDeploymentProperties() {
        synchronized(dataServiceResources) {
            Iterator i = dataServiceResources.keySet().iterator();
            while( i.hasNext() ) {
                String id = (String) i.next();
                DataResourceAccessor d = ((DataServiceResource)dataServiceResources.get(id)).getDataResourceAccessor();
                if ( d instanceof DAIXDataResourceAccessor ) 
                    ((DAIXDataResourceAccessor)d).setDeploymentProperties(deploymentProperties);
            }
        }
    }
    
    /*
     * Retrieves a data service resource accessor using the abstract name
     * contained within a resolve request.
     */
    private DataResourceAccessor getDataResourceAccessor(ResolveRequest request) throws InvalidResourceNameFaultType {
        DataServiceResource dsr = getDataServiceResource(request);
        return dsr.getDataResourceAccessor();
    }
    
    /*
     * Retrieves a data service resource accessor using the abstract name
     * contained within a request message.
     */
    private DataResourceAccessor getDataResourceAccessor(BaseRequestType request) throws InvalidResourceNameFaultType {
        DataServiceResource dsr = getDataServiceResource(request);
        return dsr.getDataResourceAccessor();
    }
    
    /**
     * Implements XUpdateAccess::XUpdateExecute
     */
    public XUpdateExecuteResponse XUpdateExecute(XUpdateExecuteRequest XUpdateExecuteRequest) throws RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, XUpdateFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        DataServiceResource resource = getDataServiceResource(XUpdateExecuteRequest);
        return queryPT.XUpdateExecute(XUpdateExecuteRequest,resource);
    }
    
    /**
     * Implements XQueryAccess::XQueryExecute
     */
    public XQueryExecuteResponse XQueryExecute(XQueryExecuteRequest XQueryExecuteRequest) throws RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, XQueryFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        DataServiceResource resource = getDataServiceResource(XQueryExecuteRequest);
        return queryPT.XQueryExecute(XQueryExecuteRequest,resource);
    }
    
    /**
     * Implements XQueryFactory::XQueryExecuteFactory
     */
    public DataResourceAddressType XQueryExecuteFactory(XQueryExecuteFactoryRequest XQueryExecuteFactoryRequest) throws RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        DataServiceResource resource = getDataServiceResource(XQueryExecuteFactoryRequest);
        return queryPT.XQueryExecuteFactory(XQueryExecuteFactoryRequest,resource);
    }
    
}
