/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.perform;

import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Element;

/**
 * 
 * Encapsulates a perform document used to execute
 * XPath activities.
 * 
 * @author Steven Lynden
 *
 */
public class XPathExecutePerformDocument extends PerformDocument {

    /**
     * Name of the OGSA-DAI activity 
     */
    public static String name = "xPathStatement";
    
    /**
     * Creates an XPath perform document to be executed against a specific resource
     * in a named collection. Note that the resource may be set to NULL, in which 
     * case the query is executed against the entire collection.
     */
    public XPathExecutePerformDocument(String collection, String resource, String[][] namespaces, String expression, String output ) throws ParserConfigurationException {
        this(name,collection,resource,namespaces,expression,output);   
    }
    
    /**
     * Creates an XPath perform document with the given activity name. This
     * method executes an XPath activity, but can also be used to execute an XQuery activity
     * by specifying the XQuery activity name.
     */
    public XPathExecutePerformDocument(String name, String collection, String resource, String[][] namespaces, String expression, String output ) throws ParserConfigurationException {
        super(name);
        //collection element
        if ( collection != null ) {
            Element collectionElem = createElementText("collection",collection);
            activity.appendChild(collectionElem);
        }
        //resource element
        if ( resource != null ) {
            Element resourceElem = createElementText("resourceId",resource);
            activity.appendChild(resourceElem);
        }
        //namespaces
        if ( namespaces != null ) {
            for(int i=0;i<namespaces.length;i++) {
                Element nsElem = createElementText("namespace",namespaces[i][1]);
                nsElem.setAttribute("prefix",namespaces[i][0]);
                activity.appendChild(nsElem);
            }
        }
        //expression
        Element expressionElem = createElementText("expression",expression);
        activity.appendChild(expressionElem);
        //output
        Element outputElem = createResourceSetElement(output);
        activity.appendChild(outputElem);
    }
    
}
