/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.perform;

import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Element;

/**
 * 
 * Encapsulates a perform document used to execute
 * XPath activities.
 * 
 * @author Steven Lynden
 *
 */
public class XUpdatePerformDocument extends PerformDocument {

    /**
     * Name of the OGSA-DAI activity 
     */
    public static String name = "xUpdateStatement";
    
    /**
     * Creates an XUpdate perform document. The resource may be specified as
     * NULL in which case the activity is executed against the named collection. 
     */
    public XUpdatePerformDocument(String collection, String resource, Element expression, String output ) throws ParserConfigurationException {
        this(name,collection,resource,expression,output);   
    }
    
    private XUpdatePerformDocument(String name, String collection, String resource, Element expression, String output ) throws ParserConfigurationException {
        super(name);
        //collection element
        if ( collection != null ) {
            Element collectionElem = createElementText("collection",collection);
            activity.appendChild(collectionElem);
        }
        //resource element
        if ( resource != null ) {
            Element resourceElem = createElementText("resourceId",resource);
            activity.appendChild(resourceElem);
        }
        //expression
        Element expressionElem = createElement("expression");
        expressionElem.appendChild(perform.importNode(expression,true));
        activity.appendChild(expressionElem);
        //output
        Element outputElem = createOutputStreamElement(output);
        activity.appendChild(outputElem);
    }
    
}
