/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.porttypes;

import java.rmi.RemoteException;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import uk.org.ogsadai.common.UniqueName;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationDocumentType;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.DataResourceUnavailableFaultType;
import uk.org.ogsadai.dais.xml.service.core.DatasetDataType;
import uk.org.ogsadai.dais.xml.service.core.DatasetType;
import uk.org.ogsadai.dais.xml.service.core.InvalidConfigurationDocumentFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidDatasetFormatFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidLanguageFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidPortTypeQNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.InvalidResourceNameFaultType;
import uk.org.ogsadai.dais.xml.service.core.NotAuthorizedFaultType;
import uk.org.ogsadai.dais.xml.service.core.ServiceBusyFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XPathExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XPathFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteFactoryRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XQueryExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XQueryFaultType;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteRequest;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateExecuteResponse;
import uk.org.ogsadai.dais.xml.services.daix.XUpdateFaultType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.service.resource.DataServiceResource;
import uk.org.ogsadai.service.resource.config.FileBasedActivitiesConfig;
import uk.org.ogsadai.service.resource.factory.DataServiceResourceFactory;
import uk.org.ogsadai.sessions.SessionConfigurationLoader;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;
import uk.org.ogsadai.wsdaix.perform.PerformDocument;
import uk.org.ogsadai.wsdaix.perform.XPathExecutePerformDocument;
import uk.org.ogsadai.wsdaix.perform.XQueryExecutePerformDocument;
import uk.org.ogsadai.wsdaix.perform.XUpdatePerformDocument;
import uk.org.ogsadai.wsdaix.resource.DAIXXMLDBResource;
import uk.org.ogsadai.wsdaix.resource.ParentXMLResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.TransientXMLDocumentResource;
import uk.org.ogsadai.wsdaix.resource.XMLCollectionResource;
import uk.org.ogsadai.wsdaix.resource.XMLSequenceResource;

/**
 * 
 * Implements the XPath, XQuery and XUpdate port types
 * 
 * @author Steven Lynden
 *
 */
public class XPathXQueryXUpdatePT extends PortTypeImpl {

    private static Logger mLog = Logger.getLogger(XPathXQueryXUpdatePT.class);
    
    /**
     * Initialse the port type
     */
    public XPathXQueryXUpdatePT(DataServiceResourceFactory factory, DeploymentProperties deploymentProperties) {
        super(factory,deploymentProperties);
    }
    
    /**
     * Implementation of XPathAccess::XPathExecute
     */
    public XPathExecuteResponse XPathExecute(XPathExecuteRequest XPathExecuteRequest, DataServiceResource resource) throws RemoteException, XPathFaultType, ServiceBusyFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        //check that the expression is valid
        URI language = XPathExecuteRequest.getXPathExpression().getLanguage();
        if ( language != null ) {
            if ( ! language.equals(DAIXProperties.XQueryXURI) )
                throw new InvalidLanguageFaultType();
        }
        //validate the dataset format URI if specified
        if ( XPathExecuteRequest.getDatasetFormatURI() != null ) {
            if ( ! XPathExecuteRequest.getDatasetFormatURI().equals(DAIXProperties.defaultDataSetFormatURI) )
                throw new InvalidDatasetFormatFaultType();
        }
        MessageElement elem = XPathExecuteRequest.getXPathExpression().get_any()[0];
        String expression = null;
        try {
            expression = DAIXUtil.XPathXToXQuery(elem.getAsDocument(),deploymentProperties.getXQueryXTransformFilename());
        } catch ( Exception e ) {
            mLog.error("Unable to convert expression to XQueryX: " + e);
            throw new XPathFaultType();   
        }
        mLog.debug("XQuery transformed to XPath Expression: " + expression);
        XPathExecuteResponse r = new XPathExecuteResponse();
        DatasetType dataType = new DatasetType();
        DatasetDataType data = new DatasetDataType();
        Document results = xPathQuery(expression,resource);
        dataType.setDatasetFormatURI( DAIXProperties.defaultDataSetFormatURI );
        MessageElement resultsElem = DAIXUtil.getResultsAsMessageElement(DAIXProperties.defaultDataSetFormatURI,results);
        data.set_any(new MessageElement[]{resultsElem});
        dataType.setDatasetData(data);
        r.setDataset(dataType);
        return r;        
    }
    
    /**
     * Constructs and XQuery perform document, executes it,
     * and returns the result
     */
    private Document xQueryQuery(String expression, DataServiceResource resource) throws XQueryFaultType {
        String outputName = "XQueryStatement-" + UniqueName.getInstance().getName();
        PerformDocument performDoc = null;
        try {
            performDoc = new XQueryExecutePerformDocument(null,null,null,expression,outputName);
        } catch ( ParserConfigurationException ex ) {
            throw new XQueryFaultType();   
        }
        try {
            return query(performDoc,resource,outputName);
        } catch ( DAIException e ) {
            mLog.error("Unable to execute perform document: " + e);
        }
        throw new XQueryFaultType();
    }
    
    /**
     * Constructs and XPath perform document, executes it,
     * and returns the result
     */
    private Document xPathQuery(String expression, DataServiceResource resource) throws XPathFaultType {
        String outputName = "XPathStatement-" + UniqueName.getInstance().getName();
        PerformDocument performDoc = null;
        try {
            performDoc = new XPathExecutePerformDocument(null,null,null,expression,outputName);
        } catch ( ParserConfigurationException ex ) {
            throw new XPathFaultType();   
        }
        try {
            return query(performDoc,resource,outputName);
        } catch ( DAIException e ) {
            mLog.error("Unable to execute perform document: " + e);
        }
        throw new XPathFaultType();
    }
    
    /**
     * A generic query method used by both XPath/XQuery to submit the
     * perform document and get the results as an XML document
     */ 
    private Document query(PerformDocument performDocument, DataServiceResource resource, String outputName) throws DAIException {
        Document responseDoc = perform(resource,performDocument);
        return PerformDocument.getResultsAsDocument(responseDoc,outputName);
    }
    
    /**
     * Implementation of XPathFactory::XPathExecuteFactory
     */
    public DataResourceAddressType XPathExecuteFactory(XPathExecuteFactoryRequest XPathExecuteFactoryRequest, DataServiceResource resource) throws RemoteException, InvalidConfigurationDocumentFaultType, XPathFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType, DataResourceUnavailableFaultType {
        ParentXMLResource dra = (ParentXMLResource) resource.getDataResourceAccessor();
        //check that the expression is valid
        URI language = XPathExecuteFactoryRequest.getXPathExpression().getLanguage();
        if ( language != null ) {
            if ( ! language.equals(DAIXProperties.XQueryXURI) )
                throw new InvalidLanguageFaultType();
        }
        ConfigurationMapType configurationMap = 
            getConfigurationMap(dra,DAIXProperties.XPathExecuteFactory,XPathExecuteFactoryRequest);
        if ( XPathExecuteFactoryRequest.getConfigurationDocument() != null ) {
            ConfigurationDocumentType configDoc = null;
            if ( ! ( XPathExecuteFactoryRequest.getConfigurationDocument() instanceof ConfigurationDocumentType ) )
                throw new InvalidConfigurationDocumentFaultType();
        }
        MessageElement elem = XPathExecuteFactoryRequest.getXPathExpression().get_any()[0];
        String expression = null;
        //determine the endpoint through which the created resource may be accessed
        String url = 
            deploymentProperties.getEndpoint( DAIXProperties.SEQUENCE_SERVICE, configurationMap.getPortTypeQName() );
        try {
            expression = DAIXUtil.XPathXToXQuery(elem.getAsDocument(),deploymentProperties.getXQueryXTransformFilename());
        } catch ( Exception e ) {
            mLog.error("Unable to convert expression to XQueryX: " + e);
            throw new XPathFaultType();   
        }
        mLog.debug("XQueryX transformed to XPath Expression: " + expression);
        //create sequence resource
        Document results = xPathQuery(expression,resource);
        DataResourceAddressType addr = null;        
        try {
            XMLSequenceResource childDRA = new XMLSequenceResource(dra,results);
            FileBasedActivitiesConfig activityConfig = DeploymentProperties.getActivityConfig(dra.getDirectory(),"");
            SessionConfigurationLoader sessionLoader = DeploymentProperties.getSessionConfig(dra.getDirectory(),"");
            DataResourceAddressType parent = null;
            if ( (dra instanceof TransientXMLCollectionResource) || (dra instanceof XMLCollectionResource ) )
                parent = DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XPATH_ACCESS_ENDPOINT),resource.getID().toString());
            else if ( (dra instanceof TransientXMLDocumentResource) )
                parent = DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XPATH_ACCESS_ENDPOINT),resource.getID().toString());
            else mLog.error("Unable to get parent resource type of:" + dra.toString() );
            String resourceId = this.exposeResource(childDRA, activityConfig, sessionLoader, deploymentProperties, parent);
            //if ( url == null ) System.out.println("url is null");
            addr = DAIXUtil.constructDataResourceAddressType(url,resourceId);
        } catch ( DAIException e ) {
            mLog.error("Unable to create XML sequence resource: " + e);
            throw new DataResourceUnavailableFaultType();
        }
        return addr;
    }
    
    /**
     * Implementation of XQueryAccess::XQueryExecute
     */
    public XQueryExecuteResponse XQueryExecute(XQueryExecuteRequest XQueryExecuteRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, XQueryFaultType, InvalidResourceNameFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        //check that the expression is valid
        URI language = XQueryExecuteRequest.getXQueryExpression().getLanguage();
        if ( language != null ) {
            if ( ! language.equals(DAIXProperties.XQueryXURI) )
                throw new InvalidLanguageFaultType();
        }
        //validate the dataset format URI if specified
        if ( XQueryExecuteRequest.getDatasetFormatURI() != null ) {
            if ( ! XQueryExecuteRequest.getDatasetFormatURI().equals(DAIXProperties.defaultDataSetFormatURI) )
                throw new InvalidDatasetFormatFaultType();
        }
        //parse the XQueryX-wrapped XPath expression
        MessageElement elem = XQueryExecuteRequest.getXQueryExpression().get_any()[0];
        String expression = null;
        try {
            expression = DAIXUtil.XQueryXToXQuery(elem.getAsDocument(),deploymentProperties.getXQueryXTransformFilename());
        } catch ( Exception e ) {
            mLog.error("Unable to convert expression to XQueryX: " + e);
            throw new XPathFaultType();   
        }
        mLog.debug("XQueryX transformed to XQuery Expression: " + expression);
        XQueryExecuteResponse r = new XQueryExecuteResponse();
        DatasetType dataType = new DatasetType();
        DatasetDataType data = new DatasetDataType();
        Document results = xQueryQuery(expression,resource);
        dataType.setDatasetFormatURI( DAIXProperties.defaultDataSetFormatURI );
        MessageElement resultsElem = DAIXUtil.getResultsAsMessageElement(DAIXProperties.defaultDataSetFormatURI,results);
        data.set_any(new MessageElement[]{resultsElem});
        dataType.setDatasetData(data);
        r.setDataset(dataType);
        return r;      
    }
    
    /**
     * Implementation of XQueryFactory::XQueryExecuteFactory
     */
    public DataResourceAddressType XQueryExecuteFactory(XQueryExecuteFactoryRequest XQueryExecuteFactoryRequest, DataServiceResource resource) throws RemoteException, InvalidConfigurationDocumentFaultType, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidPortTypeQNameFaultType, XQueryFaultType, InvalidResourceNameFaultType, NotAuthorizedFaultType {
        ParentXMLResource dra = (ParentXMLResource) resource.getDataResourceAccessor();
        //check that the expression is valid
        URI language = XQueryExecuteFactoryRequest.getXQueryExpression().getLanguage();
        if ( language != null ) {
            if ( ! language.equals(DAIXProperties.XQueryXURI) )
                throw new InvalidLanguageFaultType();
        }
        ConfigurationMapType configurationMap = 
            getConfigurationMap(dra,DAIXProperties.XQueryExecuteFactory,XQueryExecuteFactoryRequest);
        if ( XQueryExecuteFactoryRequest.getConfigurationDocument() != null ) {
            ConfigurationDocumentType configDoc = null;
            if ( ! ( XQueryExecuteFactoryRequest.getConfigurationDocument() instanceof ConfigurationDocumentType ) )
                throw new InvalidConfigurationDocumentFaultType();
        }
        MessageElement elem = XQueryExecuteFactoryRequest.getXQueryExpression().get_any()[0];
        //determine the endpoint through which the created resource may be accessed
        String url = 
            deploymentProperties.getEndpoint( DAIXProperties.SEQUENCE_SERVICE, configurationMap.getPortTypeQName() );
        String expression = null;
        try {
            expression = DAIXUtil.XQueryXToXQuery(elem.getAsDocument(),deploymentProperties.getXQueryXTransformFilename());
        } catch ( Exception e ) {
            mLog.error("Unable to convert expression to XQueryX: " + e);
            throw new XPathFaultType();   
        }
        mLog.debug("XQueryX transformed to XQuery Expression: " + expression);
        //create sequence resource
        Document results = xQueryQuery(expression,resource);
        DataResourceAddressType addr = null;        
        try {
            XMLSequenceResource childDRA = new XMLSequenceResource(dra,results);
            FileBasedActivitiesConfig activityConfig = DeploymentProperties.getActivityConfig(dra.getDirectory(),"");
            SessionConfigurationLoader sessionLoader = DeploymentProperties.getSessionConfig(dra.getDirectory(),"");
            DataResourceAddressType parent = null;
            if ( (dra instanceof TransientXMLCollectionResource) || (dra instanceof XMLCollectionResource ) )
                parent = DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.COLLECTION_XQUERY_ACCESS_ENDPOINT),resource.getID().toString());
            else if ( (dra instanceof TransientXMLDocumentResource) )
                parent = DAIXUtil.constructDataResourceAddressType(deploymentProperties.getProperty(DeploymentProperties.DOCUMENT_XQUERY_ACCESS_ENDPOINT),resource.getID().toString());
            else mLog.error("Unable to get parent resource type of:" + dra.toString() );
            String resourceId = this.exposeResource(childDRA, activityConfig, sessionLoader, deploymentProperties, parent);
            addr = DAIXUtil.constructDataResourceAddressType( url ,resourceId);
        } catch ( DAIException e ) {
            mLog.error("Unable to create XML sequence resource: " + e);
            throw new DataResourceUnavailableFaultType();
        }
        return addr;
    }
    
    /**
     * Implementation of XUpdateAccess::XUpdateExecute
     */
    public XUpdateExecuteResponse XUpdateExecute(XUpdateExecuteRequest XUpdateExecuteRequest, DataServiceResource resource) throws RemoteException, ServiceBusyFaultType, InvalidLanguageFaultType, InvalidResourceNameFaultType, XUpdateFaultType, InvalidDatasetFormatFaultType, NotAuthorizedFaultType {
        if ( ! ((DAIXXMLDBResource)resource.getDataResourceAccessor()).isWriteable() )
            throw new NotAuthorizedFaultType();
        //check that the expression is valid
        Element expression = null;
        try {
            URI language = XUpdateExecuteRequest.getXUpdateExpression().getLanguage();
            if ( language != null ) {
                if ( ! language.equals(DAIXProperties.XUpdateURI) )
                    throw new InvalidLanguageFaultType();
            }
            expression = XUpdateExecuteRequest.getXUpdateExpression().get_any()[0].getAsDOM();    
        } catch ( Exception e ) {
            throw new InvalidLanguageFaultType();
        }
        mLog.debug("XQueryX transformed to XQuery Expression: " + expression);
        XUpdateExecuteResponse r = new XUpdateExecuteResponse();
        //execute the perform document
        String outputName = "XUpdateStatement-" + UniqueName.getInstance().getName();
        PerformDocument performDoc = null;
        try {
            performDoc = new XUpdatePerformDocument(null,null,expression,outputName);
        } catch ( ParserConfigurationException ex ) {
            throw new XUpdateFaultType();   
        }
        Document response = null;
        try {
            Document responseDoc = perform(resource,performDoc);
            response = PerformDocument.getResultsAsDocument(responseDoc,outputName);
        } catch ( DAIException e ) {
            mLog.error("Error executing perform document" + e );   
        }
        long modifiedNodes = 0;
        try {
            String m = response.getElementsByTagName("modifiedNodes").item(0).getChildNodes().item(0).getNodeValue();
            modifiedNodes = Long.parseLong(m);
        } catch ( Exception e ) {
            mLog.error("Unable to retrieve the number of modified nodes, throwing fault");
            throw new XUpdateFaultType();
        }
        r.setCount(java.math.BigInteger.valueOf(modifiedNodes));
        return r;   
    }
    
}
