/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import uk.org.ogsadai.dais.xml.service.core.DataResourceAddressType;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.wsdaix.common.DeploymentProperties;

/**
 * 
 * Base class for WS-DAIX data resource accessors. All WS-DAIX resource
 * implementations extend this class.
 * 
 * @author Steven Lynden
 *
 */
public interface DAIXDataResourceAccessor extends DataResourceAccessor {
    
    /**
     * All data resources will require access to the deployment properties
     * of WS-DAIX. This method is invoked immediately after data resources
     * are created.
     */
    public void setDeploymentProperties(DeploymentProperties deploymentProperties);
    
    /**
     * Where a resource has a parent (i.e. has been created as the result of
     * a factory operation), this method is invoked immediately after creation
     * so that the child resource knows the address of the parent. This is necessary
     * because the parent address is a resource property of a WS-DAI resource. 
     */
    public void setParentDataResource(DataResourceAddressType address);
    
    /**
     * Each WS-DAIX resource implements this method to return the WS-DAI
     * data resource property document of the resource.
     */
    public PropertyDocumentType getDataResourcePropertyDocument();

}
