/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import org.apache.log4j.Logger;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.dais.xml.service.core.ChildSensitiveToParent;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.ParentSensitiveToChild;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.dais.xml.services.daix.XMLCollectionPropertyDocumentType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;

/**
 * 
 * Encapsulates a service managed XML collection resource.
 * 
 * @author Steven Lynden
 *
 */
public class TransientXMLCollectionResource extends TransientXMLDBResource {

    private static Logger mLog = Logger.getLogger(TransientXMLCollectionResource.class);
    
    /*
     * Name of the data resource
     */
    String dataResourceName;
    
    /*
     * OGSA-DAI data resource properties
     */
    private Property[] properties;
    
    public void setResourceName(String dataResourceName) {
        super.setResourceName(dataResourceName);
        this.dataResourceName = dataResourceName; 
    }
    
    /**
     * Initialise the resource
     */
    public TransientXMLCollectionResource(ParentXMLResource parent, String collectionURI, String rootCollectionURI) throws DAIException {
        super.restoreFromConfig(new java.io.File(parent.getDirectory()));
        this.mCollectionURI = collectionURI;
        this.rootCollectionURI = rootCollectionURI; 
        this.mDriverClassName = parent.getDriverClassName();
        this.mRoleMapper = parent.getRoleMapper();
        this.directory = parent.getDirectory();
        this.properties = new Property[0];
    }
    
    /**
     * Returns the OGSA-DAI resource properties associated
     * with the resource.
     */
    public Property[] getProperties() {
        return this.properties;   
    }
    
    /**
     * Returns the WS-DAI property document
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = super.getDataResourcePropertyDocument();
        ConfigurationMapType[] configurationMap = DAIXUtil.createCollectionResourceConfigurationMap();
        properties.setConfigurationMap(configurationMap);
        properties.setChildSensitiveToParent(ChildSensitiveToParent.Sensitive);
        properties.setParentSensitiveToChild(ParentSensitiveToChild.Sensitive);
        properties.setParentDataResource(parentAddress);
        return properties;
    }
    
    /**
     * Returns the WS-DAIX collection property document
     */
    public XMLCollectionPropertyDocumentType getCollectionPropertyDocument() {
        mLog.debug("Extracting xml collection property document");
        PropertyDocumentType propertyDoc = getDataResourcePropertyDocument();
        Property property = getCollectionSchemaProperty();
        return DAIXUtil.propertyDocumentToCollectionPropertyDocument(propertyDoc,property,collectionAccessor);
    }
    
}
