/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;
import uk.org.ogsadai.common.rolemap.RoleMapper;
import uk.org.ogsadai.dais.xml.service.core.DataResourceManagement;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;

/**
 * 
 * Base class for service managed XMLDB resources
 * 
 * @author Steven Lynden
 *
 */
public abstract class TransientXMLDBResource extends DAIXXMLDBResource implements ParentXMLResource {

    /*
     * The name of the data resource.
     */
    protected String dataResourceName;
    
    /*
     * The directory in which OGSA-DAI configuration files
     * for the data resource can be found. This directory is
     * inherited from the parent data resource used to create
     * the resource.
     */
    protected String directory;
    
    /**
     * Returns the OGSA-DAI role mapper object.
     */
    public RoleMapper getRoleMapper() {
        return this.mRoleMapper;   
    }
    
    /**
     * Returns the classname of the driver
     * used to interface with the database.
     */
    public String getDriverClassName() {
        return this.mDriverClassName;   
    }
    
    /**
     * Returns the directory in which configuration
     * files reside.
     */
    public String getDirectory() {
        return this.directory;   
    }
    
    /**
     * Returns the collection URI used by OGSA-DAI. 
     */
    public String getCollectionURI() {
        return this.mCollectionURI;   
    }
    
    /**
     * Returns the root collection URI used by OGSA-DAI.
     */
    public String getRootCollectionURI() {
        return this.rootCollectionURI;   
    }
    
    /**
     * Returns a property document common to all service managed
     * XMLDB resources 
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = super.getDataResourcePropertyDocument();
        properties.setDataResourceManagement(DataResourceManagement.ServiceManaged);
        return properties;
    }
    
    
}
