/*
 *  Copyright 2006 The University of Manchester
 */
package uk.org.ogsadai.wsdaix.resource;

import org.apache.log4j.Logger;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.dais.xml.service.core.ChildSensitiveToParent;
import uk.org.ogsadai.dais.xml.service.core.ConfigurationMapType;
import uk.org.ogsadai.dais.xml.service.core.ParentSensitiveToChild;
import uk.org.ogsadai.dais.xml.service.core.PropertyDocumentType;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.wsdaix.common.DAIXProperties;
import uk.org.ogsadai.wsdaix.common.DAIXUtil;

/**
 * 
 * Encapsulates a service managed XML document resource.
 * 
 * @author Steven Lynden
 *
 */
public class TransientXMLDocumentResource extends TransientXMLDBResource {

    private static Logger mLog = Logger.getLogger(TransientXMLDocumentResource.class);

    /**
     * OGSA-DAI resource ID.
     */
    private String resourceID;
    
    /**
     * OGSA-DAI data resource properties.
     */
    private Property[] properties;
    
    public void setResourceName(String dataResourceName) {
        super.setResourceName(dataResourceName);
        this.dataResourceName = dataResourceName; 
    }
    
    /**
     * Initialise the resource
     */
    public TransientXMLDocumentResource(ParentXMLResource parent, String collectionURI, String resourceID, String rootCollectionURI) throws DAIException {
        this.mCollectionURI = collectionURI;
        this.mDriverClassName = parent.getDriverClassName();
        this.mRoleMapper = parent.getRoleMapper();
        this.resourceID = resourceID;
        this.directory = parent.getDirectory();
        this.properties = new Property[0];
        this.rootCollectionURI = rootCollectionURI;
    }
    
    /**
     * Return the OGSA-DAI resource properties
     * associated with the resource.
     */
    public Property[] getProperties() {
        return this.properties;   
    }
    
    /**
     * Returns the WS-DAI property document
     */
    public PropertyDocumentType getDataResourcePropertyDocument() {
        PropertyDocumentType properties = super.getDataResourcePropertyDocument();
        ConfigurationMapType[] configurationMap = new ConfigurationMapType[2];
        configurationMap[0] = DAIXUtil.getConfigurationMap(DAIXProperties.XPathExecuteFactory,DAIXProperties.XMLSequenceAccessPT,DAIXUtil.getConfigurationDocument(false,false,false));
        configurationMap[1] = DAIXUtil.getConfigurationMap(DAIXProperties.XQueryExecuteFactory,DAIXProperties.XMLSequenceAccessPT,DAIXUtil.getConfigurationDocument(false,false,false));
        properties.setConfigurationMap(configurationMap);
        properties.setChildSensitiveToParent(ChildSensitiveToParent.Sensitive);
        properties.setParentSensitiveToChild(ParentSensitiveToChild.Sensitive);
        properties.setParentDataResource(parentAddress);
        return properties;
    }
    
}
